<?php
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

/**
 * Get readable file permissions
 * @param type $path
 */
function get_file_permissions($path)
{
    $info = null;
    if (file_exists($path))
    {
        $perms = fileperms($path);
        switch ($perms & 0xF000) {
            case 0xC000: // socket
                $info = 's';
                break;
            case 0xA000: // symbolic link
                $info = 'l';
                break;
            case 0x8000: // regular
                $info = 'r';
                break;
            case 0x6000: // block special
                $info = 'b';
                break;
            case 0x4000: // directory
                $info = 'd';
                break;
            case 0x2000: // character special
                $info = 'c';
                break;
            case 0x1000: // FIFO pipe
                $info = 'p';
                break;
            default: // unknown
                $info = 'u';
        }

        // Owner
        $info .= (($perms & 0x0100) ? 'r' : '-');
        $info .= (($perms & 0x0080) ? 'w' : '-');
        $info .= (($perms & 0x0040) ?
                    (($perms & 0x0800) ? 's' : 'x' ) :
                    (($perms & 0x0800) ? 'S' : '-'));

        // Group
        $info .= (($perms & 0x0020) ? 'r' : '-');
        $info .= (($perms & 0x0010) ? 'w' : '-');
        $info .= (($perms & 0x0008) ?
                    (($perms & 0x0400) ? 's' : 'x' ) :
                    (($perms & 0x0400) ? 'S' : '-'));

        // World
        $info .= (($perms & 0x0004) ? 'r' : '-');
        $info .= (($perms & 0x0002) ? 'w' : '-');
        $info .= (($perms & 0x0001) ?
                    (($perms & 0x0200) ? 't' : 'x' ) :
                    (($perms & 0x0200) ? 'T' : '-'));
        
    }
    return $info;
}

/**
 * Return URL encode path
 * @param string $path
 */
function ___u($path)
{
    return implode("/", array_map("rawurlencode", explode("/", $path)));
}

/**
 * Return the count of an array without emitting a PHP warning (PHP 7.2 compat)
 * @param array|\Countable $object
 * @return int
 */
function ___c($object)
{
    return (is_array($object) || ($object instanceof \Countable)) ? count ($object) : ($object instanceof \Traversable ? iterator_count($object)  : NULL );
}

/**
 * Raw URL encode a path
 * @param string $url
 * @return string
 */
function encode_path($url)
{
    $part = parse_url($url);
    //explode(',',$part['path']);
    return join('/', array_map('rawurlencode', explode('/', $part['path'])));
    
}


/**
 * Raw URL decode a path
 * @param string $url
 * @return string
 */
function decode_path($url)
{
    $part = parse_url($url);
    //explode(',',$part['path']);
    return join('/', array_map('rawurldecode', explode('/', $part['path'])));
    
}

/**
 * Get remote file content with CURL
 * @param string $url
 * @param bool $use_default_user_agent 
 * @param bool $ignore_ssl_ca
 * @return string
 */
function get_remote_file_content($url, $use_default_user_agent = true, $ignore_ssl_ca = false){
    if (function_exists('curl_init'))
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        if ($use_default_user_agent)
            curl_setopt($ch, CURLOPT_USERAGENT, 'SCHLIX/1.0 '.$_SERVER['SERVER_NAME'] );
        // For Windows only - dev server 
        // note: do not use SCHLIX_SYSTEM_MSFTWIN here in case it's used during install
        if ($ignore_ssl_ca || strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);    
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_BUFFERSIZE, 64000);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);       
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT , 10); 
        $data = curl_exec($ch);
        $err = curl_error($ch);
        $retcode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
        curl_close($ch);
        if ($err)
            throw new Exception(sprintf(___('Error while downloading file: %s'), $err));        
        if ($retcode != 200)
            throw new Exception(sprintf(___('Unable to download the requested file. HTTP Error Code: %d'), $retcode));
    } else
    {
        $context_opts =[
            "ssl"=>[
                'allow_self_signed' => true,
                'verify_peer'=>  true,
                "verify_peer_name"=> false,
            ]];
        if (!ini_get('allow_url_fopen'))
            @ini_set('allow_url_fopen', 1);
        $data = ($ignore_ssl_ca || strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? 
                @file_get_contents($url, false, stream_context_create($context_opts)) :
                @file_get_contents($url);
    }
    return $data;
}

/**
 * Start output buffering
 */
function start_output_buffer($clean_first = false)
{
    if ($clean_first && ob_get_length() > 0)
        ob_end_clean();
    ob_start();   
}

/**
 * End output buffering and return string
 * @return string
 */
function end_output_buffer()
{
    $text = ob_get_contents();
    ob_end_clean();
    return $text;
}
/**
 * String output sanitizing function to ensure there's no XSS in the text.
 * If maxlen is specified then it will be limited to maxlen
 * @param string $s
 * @param int $maxlen
 * @return string
 */
function ___h($s, $maxlen = 0) {
    if ($maxlen > 0 && strlen($s) > $maxlen) {
        $s = substr($s, 0, $maxlen);
    }
    // htmlspecialchars($s, ENT_QUOTES);    
    return htmlentities($s, ENT_QUOTES, "UTF-8"); 
}
/**
 * Alias of ___h/ Deprecated for an easier to remember function name. Use ___h. 
 * String output sanitizing function to ensure there's no XSS in the text.
 * If maxlen is specified then it will be limited to maxlen.
 * To be removed in v2.3.x-x
 * @deprecated since version 2.1.6-2
 * @param string $s
 * @param int $maxlen
 * @return string
 */
function SAFE_HTML($s, $maxlen = 0) {
    return ___h($s, $maxlen);
}

/**
 * Converts a datetime or a UNIX date and return it in the format of year-month-day time
 * @param string|date $the_date
 * @param bool $display_time
 * @return string
 */
function format_nice_date($the_date, $display_time = false) {
    $date = is_integer($the_date) ? $date : strtotime($the_date);
    $year = '<span class="year">' . date('Y', $date) . '</span>';
    $month = '<span class="month">' . date('M', $date) . '</span>';
    $day = '<span class="day">' . date('d', $date) . '</span>';
    $time = ($display_time) ? '<span class="time">' . date('H:i:s', $date) . '</span>' : '';
    //'Y-m-d H:i:s'
    return "{$year}{$month}{$day}{$time}";
}

/**
 * http://stackoverflow.com/questions/18685/how-to-display-12-minutes-ago-etc-in-a-php-webpage
 * @param int $time_since
 * @return string
 */
function time_since($time_since) {
    $time = time() - $time_since;
    $tokens = array(
      31536000 => ___('year'),
      2592000 => ___('month'),
      604800 => ___('week'),
      86400 => ___('day'),
      3600 => ___('hour'),
      60 => ___('minute'),
      1 => ___('second')
    );
    $extra_s = (SCHLIX_DEFAULT_LANGUAGE !== 'en_us' && SCHLIX_DEFAULT_LANGUAGE !== 'en-CA' && SCHLIX_DEFAULT_LANGUAGE !== 'en-GB') ? '' : 's';
    foreach ($tokens as $unit => $text) {
        if ($time < $unit)
            continue;
        $numberOfUnits = floor($time / $unit);
        return $numberOfUnits . ' ' . $text . (($numberOfUnits > 1) ? $extra_s : '');
    }
}

/**
 * Returns true if haystack starts with needle
 * @param string $haystack
 * @param string $needle
 * @return boolean
 */
function str_starts_with($haystack, $needle) {
    return (0 === strpos($haystack, $needle));
}

/**
 * Returns true if haystack ends with needle
 * @param string $haystack
 * @param string $needle
 * @return boolean
 */

function str_ends_with($haystack, $needle)
{
    $length = strlen($needle);
    return ($length == 0) ? true :  (strrpos($haystack, $needle) == strlen($haystack) - $length);
}
/**
 * Returns true if haystack contains needle
 * @param string $haystack
 * @param string $needle
 * @return boolean
 */
function str_contains($haystack, $needle) {
    return strpos($haystack, $needle) !== FALSE;
}

/**
 * Returns string value of $str and if the length is greater than maxlen
 * then it will be trimmed
 * @param string $str
 * @param int $maxlen
 * @return string
 */
function str_limit($str, $maxlen)
{
    $s = strval($str);
    if ($maxlen > 0 && strlen($s) > $maxlen) {
        $s = substr($s, 0, $maxlen);
    }
    return $s;
}

//_______________________________________________________________________________________________________________//
function get_class_ancestors($class) {
    if (is_object($class)) {
        $classes[] = get_class($class);
        while ($class = get_parent_class($class)) {
            $classes[] = $class;
        }
        return $classes;
    } else {
        // echo 'Invalid '.$class;
        return [];
    }
}

//_______________________________________________________________________________________________________________//
/**
 * Returns a user's real IP address. Probes for proxy if the CMS is behind
 * a load balancer.
 * @return string
 */
function get_user_real_ip_address() {
    if (isset($_SERVER["HTTP_CF_CONNECTING_IP"]))
        return $_SERVER['HTTP_CF_CONNECTING_IP'];
    if (isset($_SERVER['HTTP_CLIENT_IP']))
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    else
        $ip = $_SERVER['REMOTE_ADDR'];
    

    if (strstr($ip, ',')) {
        $ips = explode(',', $ip);
        $ip = trim($ips[0]);
    }    
    return $ip;
}

function get_system_or_user_file($filename, $die = true) {
    $sys_file = remove_multiple_slashes(SCHLIX_SYSTEM_PATH . '/' . $filename);
    $user_file = remove_multiple_slashes(SCHLIX_SITE_PATH . '/' . $filename);

    if (file_exists($user_file))
        return $user_file;
    elseif (file_exists($sys_file))
        return $sys_file;
    elseif ($die)
        die("File {$filename} cannot be found anywhere");
}

function get_system_or_user_url_path($filename, $die = true) {
    $user_file = remove_multiple_slashes(SCHLIX_SITE_PATH . '/' . $filename);
    $sys_file = remove_multiple_slashes(SCHLIX_SYSTEM_PATH . $filename);
    if (file_exists($$user_file))
        return remove_multiple_slashes('/' . $filename);
    elseif (file_exists($sys_file))
        return remove_multiple_slashes('/system/' . $filename);
    elseif ($die != false)
        die("File {$filename} cannot be found anywhere, URL not generated");
}

//_________________________________________________________________________//
/**
 * Returns the application real name, given the alias as defined
 * in the application configuration page
 * @global \SCHLIX\cmsDatabase $SystemDB
 * @param string $alias
 * @return string
 */
function get_application_real_name_by_alias($alias) {
    global $SystemDB;

    if (empty($alias)) return NULL;
    $sql = 'SELECT title FROM gk_app_items WHERE app_alias = ' . sanitize_string($alias);
    $result = $SystemDB->getQueryResultSingleRow($sql);
    return ($result != null) ? $result['title'] : $alias;
}
/**
 * Returns a new, non-duplicate filename 
 * @param string $folder
 * @param string $original_filename
 * @return string
 */
function prevent_duplicate_filename_in_folder($folder, $original_filename) {
    $filename = pathinfo($original_filename, PATHINFO_FILENAME);
    $extension = pathinfo($original_filename, PATHINFO_EXTENSION);
    $new_filename = $original_filename;
    $count = 2;
    while (file_exists(remove_multiple_slashes($folder . '/' . $new_filename))) {
        $new_filename = $filename . '_' . $count . '.' . $extension;
        $count++;
    }
    return $new_filename;
}

            
//_________________________________________________________________________//
/**
 * Force URL to be HTTPS if the site's SSL variable is enabled and HTTPS URL is defined
 * inthe subsite configuration
 * @return string
 */
function force_https_url() {
    return (defined('SCHLIX_SITE_HTTPS_URL') && SCHLIX_SITE_SSL_ENABLED && SCHLIX_SITE_HTTPS_URL != '') ? SCHLIX_SITE_HTTPS_URL : SCHLIX_SITE_HTTP_URL;
}

//_________________________________________________________________________//
/**
 * Setthe CSRF token if none
 * @return string
 */
function set_csrf_token() {
    $_SESSION['csrf'] = (empty($_SESSION['csrf'])) ? sha1(SCHLIX_SITE_NAME . uniqid(mt_rand(), TRUE)) : $_SESSION['csrf'];
    return $_SESSION['csrf'];
}

//_________________________________________________________________________//
/**
 * Returns the CSRF token
 * @return string
 */
function get_csrf_token() {
    return ($_SESSION['csrf']);
}

//_________________________________________________________________________//
/**
 * Check if the variable _csrftoken matches with the currently set token.
 * If the array is not specified, then $_POST will be used
 * @param array $datavalues
 * @return bool
 */
function is_valid_csrf($datavalues = NULL) {
    $csrf_token = (!is_array($datavalues) || empty($datavalues)) ? $_POST['_csrftoken'] :$datavalues['_csrftoken'];
    return $_SESSION['csrf'] === $csrf_token;
}

function check_csrf_halt_on_error($is_ajax = false) {
    if (!is_valid_csrf()) {
        if (is_ajax_request() || $is_ajax) {
            ajax_echo(ajax_reply('400', 'Invalid Ajax Request - CSRF cannot be verified'));
            die;
        } else {
            die('Invalid CSRF Verification Token');
        }
    }
}

//_________________________________________________________________________//
/**
 * Returns true if image is valid (beta)
 * @param string $tmp_filename
 * @param string $uploaded_filename
 * @return boolean
 */
function is_valid_uploaded_image_file($tmp_filename, $uploaded_filename)
{
    global $SystemLog;
    
    if (!file_exists($tmp_filename))
        return false;
    
    $allowed_exts = ['png','jpg','jpeg','gif'];
    $valid = true;
    
    $ext = strtolower( pathinfo($uploaded_filename, PATHINFO_EXTENSION));
    if (in_array($ext, $allowed_exts) === FALSE)
    {
        $SystemLog->record( $ext);
        return false;
    }
    
    $img_info = getimagesize($tmp_filename);
    $imgtype = $img_info[2];
    
    if ($imgtype > 0 && $imgtype < 4) // gif, png, jpeg
    {
        
        $sx = (int) $img_info [0];
        $sy = (int) $img_info [1];
        $valid = ($sx > 0 && $sx < 8192) && ($sy > 0 && $sy < 8192);
    }
    if ($valid && ($ext == 'gif' || $imgtype == 1))
    {
        $fstr = file_get_contents($tmp_filename);
        $valid = !str_contains($fstr,'<?php');
    }
        
    return $valid;
}


// This is a quicky , will be fixed later
//_________________________________________________________________________//
/**
 * Record 404 error and redirect
 * @global \SCHLIX\cmsDatabase $SystemDB
 * @return type
 */
function record_404_error_and_redirect() {
    global $SystemDB;

    if (SCHLIX_RECORD_404_ERRORS != 1)
        return;

    $url_proto = 'http' . (empty($_SERVER['HTTPS']) ? '' : 's') . '://';
    $url_parts = parse_url($url_proto . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI']);
    //$real_url = $url_parts['scheme'] . '://' . $url_parts['host'] . $url_parts['path'];

    $request_path = strtolower(trim($url_parts['path']));

    $sanitized_request_path = sanitize_string($request_path);
    $existing_url_data = $SystemDB->getQueryResultSingleRow("SELECT id, url_replacement FROM gk_404_error WHERE url = {$sanitized_request_path}");
    $replacement = $existing_url_data['url_replacement'];
    if ($replacement != '') {
        $SystemDB->query("UPDATE gk_404_error SET hits_redirected = hits_redirected + 1 WHERE id = {$existing_url_data['id']}");

        // if there's a replacement, then redirect
        if (strpos($replacement, "://") !== FALSE)
            redirect_url($replacement);
        else {

            redirect_url($url_proto . $url_parts['host'] . $replacement);
        }
    } else {
        $sanitized_host = sanitize_string($_SERVER['SERVER_NAME']);
        $SystemDB->query("INSERT INTO gk_404_error (request_host, url,date_modified) VALUES ({$sanitized_host},{$sanitized_request_path}, NOW()) ON DUPLICATE KEY UPDATE hits_error = hits_error + 1;");
    }
    //$existing_url =  
}

//_________________________________________________________________________//
/**
 * Display HTTP Error (e.g. 404, 400, etc). Internal use only
 * @internal
 * @param int $error_number
 */
function display_http_error($error_number) {

    $defaultHTTPErrorMessages = array(
      400 => 'HTTP/1.0 400 Bad Request',
      401 => 'HTTP/1.0 401 Unauthorized',
      402 => 'HTTP/1.0 402 Payment Required',
      403 => 'HTTP/1.0 403 Forbidden',
      404 => 'HTTP/1.0 404 Not Found',
      405 => 'HTTP/1.0 404 Method Not Allowed',
      406 => 'HTTP/1.0 404 Not Acceptable',
      407 => 'HTTP/1.0 404 Proxy Authentication Required',
      408 => 'HTTP/1.0 404 Request Time-out',
      409 => 'HTTP/1.0 404 Conflict',
      410 => 'HTTP/1.0 404 Gone',
      411 => 'HTTP/1.0 404 Length Required',
      412 => 'HTTP/1.0 404 Precondition Failed',
      413 => 'HTTP/1.0 404 Request Entity Too Large',
      414 => 'HTTP/1.0 404 Request-URI Too Large',
      415 => 'HTTP/1.0 404 Unsupported Media Type',
      500 => 'HTTP/1.0 500 Internal Server Error'
    );

    
    header($defaultHTTPErrorMessages[$error_number]);
    if (($error_number >= 400 && $error_number <= 415) || $error_number == 50) {
        if ($error_number == 404)
            record_404_error_and_redirect();
        if (!is_ajax_request()) {
            $http_error = new \App\HttpError();
            $http_error->viewItemByID($error_number);
        } else {
            ajax_echo(ajax_reply($error_number, $defaultHTTPErrorMessages[$error_number]));
        }
    }
}

//_________________________________________________________________________//
/**
 * Returns a Javascript variable that contains csrftoken string
 * e.g. var variable_name = '1231351232asdasd' 
 * @param string $varname
 * @return string
 */
function get_js_csrf_var($varname = '_csrftoken') {
    $s = 'var '.$varname.'="' . get_csrf_token() . '";';
    return $s;
}

//_________________________________________________________________________//
/**
 * Adds an error string that will be displayed when the page loads again
 * next time
 * @internal
 * @param string $error
 */
function display_schlix_alert($error) {
    if (!array_key_exists('schlix_internal_messages', $_SESSION) || !is_array($_SESSION['schlix_internal_messages']))
        $_SESSION['schlix_internal_messages'] = [];
    $add = true;
    $count = ___c($_SESSION['schlix_internal_messages']);
    if ($count > 0)
    {
        if ($_SESSION['schlix_internal_messages'][$count-1] == $error)
            $add = false;
    }
    if ($add)
        $_SESSION['schlix_internal_messages'][] = $error;
}

//_________________________________________________________________________//
/**
 * Clear error string
 */
function clear_schlix_alert() {
    unset($_SESSION['schlix_internal_messages']);
}

//_________________________________________________________________________//
/**
 * Returns true if the input string is serialized
 * @param string $string
 * @return bool
 */
function is_serialized($string) {
    return ((@unserialize($string) !== false && strpos($string, '{') !== false) || $string === 'b:0;');
}

//_______________________________________________________________________________________________________________//
/**
 * Generates a new UUIDv4
 * Source Credit:  copied and pasted this from http://php.net/manual/en/function.uniqid.php
 * @return string
 */
function new_uuid_v4() { 
    // The field names refer to RFC 4122 section 4.1.2
    return sprintf('%04x%04x-%04x-%03x4-%04x-%04x%04x%04x', mt_rand(0, 65535), mt_rand(0, 65535), // 32 bits for "time_low"
        mt_rand(0, 65535), // 16 bits for "time_mid"
        mt_rand(0, 4095), // 12 bits before the 0100 of (version) 4 for "time_hi_and_version"
        bindec(substr_replace(sprintf('%016b', mt_rand(0, 65535)), '01', 6, 2)),
        // 8 bits, the last two of which (positions 6 and 7) are 01, for "clk_seq_hi_res"
        // (hence, the 2nd hex digit after the 3rd hyphen can only be 1, 5, 9 or d)
        // 8 bits for "clk_seq_low"
        mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535)); // 48 bits for "node"
}

//_________________________________________________________________________//
function get_application_alias($app, $subclass = NULL) {
    global $SystemDB;
    
    $subclass_suffix = $subclass ? '.'.$subclass : '';
    $app_alias_cache = \SCHLIX\cmsContextCache::get('app_alias',$app);
    if ($app_alias_cache) 
        return $app_alias_cache.$subclass_suffix;
    else
    {
        $sql = 'SELECT app_alias FROM gk_app_items WHERE title = ' . sanitize_string($app);
        $result = $SystemDB->getQueryResultSingleRow($sql);
        $alias = null;
        if ($result != null) {
            $alias = $result['app_alias'];
            if (!$alias)
                $alias = $app;
        } else
            $alias = $app;
        \SCHLIX\cmsContextCache::set('app_alias',$app, $alias);
        return $alias.$subclass_suffix;
    }
}

//_________________________________________________________________________//
/**
 * Redirect URL
 * @param string $url
 */
function redirect_url($url) {
    ob_end_clean();
    ob_start();
    header("Location: {$url}");
    exit();
}

/**
 * Return URL encode
 * @param string $path
 */
function encode_url($path)
{
    return ___u($path); // implode("/", array_map("rawurlencode", explode("/", $path)));
}

//_________________________________________________________________________//
/**
 * Returns a list of all applications
 * @global type $SystemConfig
 * @param bool $use_cache
 * @param int $cache_how_many_minutes
 * @return array
 */
function get_list_of_all_apps($use_cache = false, $cache_how_many_minutes = 5) {
    global $SystemConfig;

    $refresh_period_in_second = intval($cache_how_many_minutes) * 60;
    $last_time_when_it_was_cached = intval($SystemConfig->get('system', 'frontend_apps_cache_time'));
    $cached_app_list = $SystemConfig->get('system', 'frontend_apps_cache_content');
    if ($use_cache && ((time() - $last_time_when_it_was_cached) <= $refresh_period_in_second) && (___c($cached_app_list) > 0)) {
        return $cached_app_list;
    } else {
        $main_dir = SCHLIX_SYSTEM_PATH . '/apps/';
        $app_list = [];

        $apps_dir = new DirectoryIterator($main_dir);
        if ($apps_dir)
            foreach ($apps_dir as $app_dir)
                if (!$app_dir->isDot() && $app_dir->isDir() && (strpos($app_dir->getFileName(), 'uninstalled_') === false) && file_exists($main_dir . $app_dir->getFileName() . '/' . $app_dir->getFileName() . '.class.php'))
                    $app_list[] = $app_dir->getFileName();

        $main_dir = SCHLIX_SITE_PATH . '/apps/';
        $apps_dir = new DirectoryIterator($main_dir);
        if ($apps_dir)
            foreach ($apps_dir as $app_dir)
                if (!$app_dir->isDot() && $app_dir->isDir() && (strpos($app_dir->getFileName(), 'uninstalled_') === false) && file_exists($main_dir . $app_dir->getFileName() . '/' . $app_dir->getFileName() . '.class.php'))
                    $app_list[] = $app_dir->getFileName();
        if ($use_cache) {
            $SystemConfig->set('system', 'frontend_apps_cache_time', time());
            $SystemConfig->set('system', 'frontend_apps_cache_content', $app_list);
        }
        return $app_list;
    }
}

//______________________________________________________________________________________//
/**
 * Returns true if Captcha is valid
 * @return bool
 */
function is_captcha_verification_valid() {
    $securimage = new \securimage();
    $securimage->session_name = SCHLIX_SESSION_NAME;
    return ($securimage->check($_POST['verification_code']) != false);
}

//______________________________________________________________________________________//
/**
 * Converts data:image/png;base64 from text to image
 * @global \SCHLIX\cmsLogger $SystemLog
 * @param string $txt
 * @param string $upload_path
 * @param string $url_path
 * @param string $filename_prefix
 * @return string
 */
function convert_pasted_png_images_from_html_text($txt, $upload_path, $url_path, $filename_prefix = 'img') {
    global $SystemLog;

    $filename_prefix = alpha_numeric_only($filename_prefix);
    if (get_magic_quotes_gpc())
        $txt = stripslashes($txt);
    $save_result = false;
    //preg_match_all('/src="data:image\\/png;base64[^>]+>/i',$txt, $pasted_images_array);
    preg_match_all('/src=("data:image\\/png;base64[^"]*")/i', $txt, $pasted_images_array);
    $i = 1;
    $start_tag = 'src="data:image/png;base64,';
    $end_tag = '"';
    $new_start_tag = 'src="';

    foreach ($pasted_images_array[0] as $img_tag) {
        $outputfile = $filename_prefix.'-'.date('Y-m-d') . "-{$i}.png";
        while (file_exists($upload_path . $outputfile)) {
            $outputfile = $filename_prefix.'-'.date('Y-m-d') . "-{$i}.png";
            $i++;
        }
        $imageData = str_replace($start_tag, '', $img_tag);
        $imageData = str_replace($end_tag, '', $imageData);
        $ifp = fopen($upload_path . $outputfile, "wb");
        if ($ifp) {
            $bin64_decoded_data = base64_decode($imageData);
            $save_result = fwrite($ifp, $bin64_decoded_data);
            fclose($ifp);
            $txt = str_replace($start_tag . $imageData, $new_start_tag . $url_path . $outputfile, $txt);
        }
        $log = ($save_result) ? "Fetched pasted image and saved {$save_result} bytes to {$upload_path}{$outputfile}" : "Cannot convert pasted image to {$upload_path}{$outputfile}";
        $SystemLog->record($log);
    }
    return $txt;
}

//______________________________________________________________________________________//
/**
 * Search for all external images hosted outside of this URL in this HTML text
 * to a local one. If the file already exists with the same name then it will be ignored
 * @param string $txt
 * @param string $upload_path
 * @param string $url_path
 * @return string
 */
function move_static_external_images_from_html_text($txt, $upload_path, $url_path) {
    if (get_magic_quotes_gpc())
        $txt = stripslashes($txt);
    preg_match_all('/<img[^>]+>/i', $txt, $images_array);
    $imgs = [];
    foreach ($images_array[0] as $img_tag) {
        //echo $img_tag;
        preg_match_all('/(alt|title|src)=("[^"]*")/i', $img_tag, $imgs[$img_tag]);
    }
    foreach (array_keys($imgs) as $x) {

        $src = $imgs[$x][2][0];
        $cleanurl = str_replace('"', '', $src);
        $cleanurl = str_replace("'", '', $cleanurl);
        $prs = parse_url($cleanurl);
        if (($prs['host'] != '') && strpos(SCHLIX_SITE_URL, $prs['host']) === false && ($prs['host'] != '')) {
            $basename = basename($cleanurl);
            $outputfile = $basename;
            if (!file_exists($upload_path . $outputfile)) {
                $filecontent = file_get_contents($cleanurl);
                file_put_contents($upload_path . $outputfile, $filecontent);
            }
            $txt = str_replace($cleanurl, $url_path . $outputfile, $txt);
        }
    }
    return $txt;
}

//______________________________________________________________________________________//
function quote_field_name_for_query($item_field) {
    if ($item_field) {
        $item_field = "`{$item_field}`";
    } else
        $item_field = '';
    return $item_field;
}

//______________________________________________________________________________________//
function quote_array_of_field_names_for_query($item_fields) {
    $total_field_count = ___c($item_fields);
    for ($i = 0; $i < $total_field_count; $i++)
        $item_fields[$i] = quote_field_name_for_query($item_fields[$i]);
    return $item_fields;
}

/**
 * Returns alphanumeric of string with - (dash) and _ (underscore) allowed
 * @param string $str
 * @param string $replacement
 * @return string
 */
function alpha_numeric_with_dash_underscore($str, $replacement = '-') {
    $str = preg_replace('/[^a-zA-Z0-9-_]/', $replacement, $str);
    return trim($str, ' -');
}


/**
 * Like alpha_numeric_with_dash_underscore but without - (dash) and (underscore) allowed
 * @param string $str
 * @param string $replacement
 * @return string
 */
function alpha_numeric_only($str, $replacement = '') {
    $str = preg_replace('/[^a-zA-Z0-9]/', $replacement, $str);
    return trim($str);
}

/**
 * Converts a string to safe filename
 * @param string $s
 * @return string
 */
function convert_to_safe_filename($s)
{
    return preg_replace('/[^a-zA-Z0-9-_\.\/]/','', $s);
}


/**
 * Converts a string to an alphanumeric one (dash and underscores allowed).
 * This is mainly used to convert a title page to virtual filename
 * @param string $str
 * @param bool $allow_utf8
 * @return string
 */
function convert_into_sef_friendly_title($str, $allow_utf8 = true) {
    /*  $str = htmlentities($str, ENT_QUOTES, 'UTF-8');
      $str = preg_replace('~&([a-z]{1,2})(acute|cedil|circ|grave|lig|orn|ring|slash|th|tilde|uml);~i', '$1', $str);
      $str = html_entity_decode($string, ENT_QUOTES, 'UTF-8');
     */
    $str = strtolower(trim($str));
    //   $str = preg_replace(array('~[^0-9a-z]~i', '~[ -]+~'), ' ', $str);

    $pattern = ($allow_utf8) ? '~[^\p{L}\p{N}\p{Mn}\p{Pd}\'\x{2019}_]~u' : '/[^a-z0-9_\-]/';
    
    $str = preg_replace($pattern, '-', $str);
    $str = preg_replace('/-+/', "-", $str);

    return trim($str, ' -');
}

//______________________________________________________________________________________//
/**
 * Returns the number of seconds between start_date and end_date
 * @param string|date $start_date
 * @param string|date $end_date
 * @return int
 */
function time_difference($start_date, $end_date) {
    return  strtotime($end_date) - strtotime($start_date);
}

//______________________________________________________________________________________//
/**
 * Returns the day numbers between start_date and end_date
 * @param string|date $start_date
 * @param string|date $end_date
 * @return int
 */
function days_difference($start_date, $end_date) {
    $x = strtotime($end_date) - strtotime($start_date);
    $number_of_days = floor($x / (60 * 60 * 24));
    return $number_of_days;
}

//______________________________________________________________________________________//
/**
 * Returns the number of day difference from today
 * @param string|date $the_date
 * @return int
 */
function days_difference_from_today($the_date) {
    return days_difference(get_current_datetime(), $the_date);
}

//_________________________________________________________________________//
/**
 * Return string of current date in ISO-8601 format
 * @return string
 */
function get_current_datetime() {
    return date('Y-m-d H:i:s');
}

//_________________________________________________________________________//
/**
 * Returns true if email address is DNS verified with the MX record
 * @param string $email
 * @return bool
 */
function verify_email_dns($email) {
    if (empty($email))
        return false;

    list($name, $domain) = explode('@', $email);
    return (empty($domain) || !checkdnsrr($domain, 'MX')) ? false : $email;
}

//_________________________________________________________________________//
/**
 * Select HTTP POST Variables. Useful to filter variables based on table names
 * @param string|array $str
 * @return array
 */
function select_http_post_variables(/* ... */) {
    if (!$_POST)
        return false;
    if (func_num_args() == 0)
        return $_POST;
    foreach ($_POST as $key => $value)
    {
        if (is_string($_POST[$key]))
            $_POST[$key] = trim($value);
    }
    if (is_array(func_get_arg(0)))
        $array = array_fill_keys(func_get_arg(0), NULL);
    else
        $array = array_fill_keys(func_get_args(), NULL);
    return array_intersect_key($_POST, $array);
}

/**
 * Convert array values to keys and fill it with number
 * @param array $array
 * @param mixed $fill
 * @return array
 */
function convert_array_values_to_keys($array, $fill = 1)
{
    if (!is_array($array))
        return [];
    $result = array_flip($array);
    foreach ($result as $key => $value)
        $result[$key] = $fill;
    return $result;
}
//_________________________________________________________________________//
/**
 * Remove multiple slashes from string. 
 * @param string $str
 * @return string
 */
function remove_multiple_slashes($str) {
    return !empty($str) ? preg_replace('#/{2,}#', '/', $str) : '';
}

//_________________________________________________________________________//
/**
 * Sanitize string - SQL
 * @global \SCHLIX\cmsDatabase $SystemDB
 * @param string $value
 * @param bool $dont_add_quotes
 * @return string
 */
function sanitize_string($value, $dont_add_quotes = false) {
    global $SystemDB;
    if (!is_array($value))
        $value = $SystemDB->escapeString($value);
    $value = ($dont_add_quotes ? $value : "'" . $value . "'");
    //if (!is_numeric($value)) $value = "'" . str_replace("'","''",$value). "'";
    //if (!is_numeric($value)) $value = "'" . $value. "'"; // Nov 9, 2011 - Prana fix ' bug
    //if (!is_numeric($value)) $value = "'" . addslashes($value) . "'";
    return $value;
}


/**
 * Returns true if input is in date format
 * @param object|string|DateTime $date
 * @param string $format
 * @return bool
 */
function is_date($date, $format = 'Y-m-d H:i:s') {
    $d = DateTime::createFromFormat($format, $date);
    return $d && $d->format($format) == $date;
}


//______________________________________________________________________________________//
/**
 * Returns true if this request is AJAX
 * @return bool
 */
function is_ajax_request() {
    return ((!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest'));
}

//______________________________________________________________________________________//
/**
 * Returns true if it's a postback ... similar to ASP.NET's isPostBack()
 * @return bool
 */
function is_postback() {
    return ($_SERVER['REQUEST_METHOD'] === 'POST');
}

//______________________________________________________________________________________//
/**
 * Perform a real strip tag. Function copied from php.net
 * @param string $i_html
 * @param array $i_allowedtags
 * @param bool $i_trimtext
 * @return string
 */
function real_strip_tags($i_html, $i_allowedtags = [], $i_trimtext = FALSE) {
    if (!is_array($i_allowedtags))
        $i_allowedtags = !empty($i_allowedtags) ? array($i_allowedtags) : [];
    $tags = implode('|', $i_allowedtags);

    if (empty($tags))
        $tags = '[a-z]+';

    preg_match_all('@</?\s*(' . $tags . ')(\s+[a-z_]+=(\'[^\']+\'|"[^"]+"))*\s*/?>@i', $i_html, $matches);

    $full_tags = $matches[0];
    $tag_names = $matches[1];

    foreach ($full_tags as $i => $full_tag) {
        if (!in_array($tag_names[$i], $i_allowedtags))
            if ($i_trimtext)
                unset($full_tags[$i]);
            else
                $i_html = str_replace($full_tag, '', $i_html);
    }

    return $i_trimtext ? implode('', $full_tags) : $i_html;
}

/**
 * Create directory with permission 0755 if not exist
 * @param string $dir
 * @return bool
 */
function create_generic_directory($dir, $permission = 0755, $recursive = true)
{
    if (!is_dir($dir)) {
        if (!mkdir($dir, $permission, $recursive)) {
            return false;
        }
    }
    return true;
}

/**
 * Replaces Windows \ with /
 * @param string $f
 * @return string
 */
function unix_slash($f)
{
    return $f ? str_replace('\\', '/', $f) : '';
}
/**
 * Copy recursive from source to destination
 * @param string $source
 * @param string $dest
 * @return bool
 */
function copy_recursive($source, $dest) {

    if (!is_dir($source)) 
        return false;
    create_generic_directory($dest, 0755, false);
    foreach ($iterator = new \RecursiveIteratorIterator(
    new \RecursiveDirectoryIterator($source, \RecursiveDirectoryIterator::SKIP_DOTS), \RecursiveIteratorIterator::SELF_FIRST) as $item) {
        if ($item->isDir()) {
            $target_dir = $dest . '/' . unix_slash($iterator->getSubPathName());
            create_generic_directory($target_dir, 0755, false);
        } else copy($item, $dest . '/' . unix_slash($iterator->getSubPathName()));
    }
}

/**
 * Sort an associative array by key
 * @param array $array
 * @param string $subkey
 * @param int $sortType
 */
function sort_associative_array_by_key(&$array, $subkey, $sortType = SORT_ASC) {
    if (is_array($array) && (count($array) > 0) && array_key_exists($subkey, $array[0])) {
        $keys = [];
        foreach ($array as $subarray) {
            $keys[] = $subarray[$subkey];
        }
        array_multisort($keys, $sortType, $array);
    }
}
//_________________________________________________________________________//
/**
 * Returns a JSON-encoded string of reply. Set status to 200 for ok
 * @param int $status
 * @param array $data
 * @return string
 */
function ajax_reply($status, $data) {
    header('Content-Type: application/json; charset=UTF-8');
    header("Cache-Control: no-cache, must-revalidate"); // HTTP/1.1
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // Date in the past
    $reply ['status'] = $status;
    $reply ['data'] = $data;
    return json_encode($reply);
}

//_________________________________________________________________________//
/**
 * echo ajax reply and return false (RETURN AS AJAX)
 * @param string $s
 * @return string
 */
function ajax_echo($s) {

    echo $s;
    return FALSE;
}

//_________________________________________________________________________//
/**
 * Returns a JSON-encoded array of message for datasource in SCHLIX.UI.DataTable
 * @param int $status
 * @param int $allRecords
 * @param int $start
 * @param int $end
 * @param int $item_per_page
 * @param int $total_item_count
 * @param string $sort
 * @param string $dir
 * @return string
 */
function ajax_datasource_reply($status, $allRecords, $start = 0, $end = 0, $item_per_page = 0, $total_item_count = 0, $sort = null, $dir = 'asc') {
    header('Content-Type: application/json; charset=UTF-8');
    header("Cache-Control: no-cache, must-revalidate"); // HTTP/1.1
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // Date in the past

    if ($dir != 'desc')
        $dir = 'asc';
    $record_count = ___c($allRecords);
    if ($end == 0)
        $end = $record_count;
    if ($total_item_count == 0)
        $total_item_count = $record_count;

    // Create return value
    $returnValue = array(
      'status' => intval($status),
      'recordsReturned' => intval($record_count),
      'totalRecords' => intval($total_item_count),
      'start' => intval($start),
      'end' => intval($end),
      'itemsperpage' => intval($end - $start),
      'sortby' => $sort,
      'sortdirection' => $dir,
      'data' => $allRecords
    );

    return json_encode($returnValue);
}

//_________________________________________________________________________//

function restore_mysql_backup_from_file($filename) {
    global $SystemDB;

    $templine = '';
    $lines = file($filename);
    foreach ($lines as $line) {
        //echo $line;
        if (substr($line, 0, 2) == '--' || empty($line))
            continue;
        $templine .= $line;
        // If it has a semicolon at the end, it's the end of the query
        if (substr(trim($line), -1, 1) == ';') {
            $SystemDB->query($templine);
            $templine = '';
        }
    }
}

/**
 * Returns true if the file is an animated gif
 * @param string $filename
 * @return boolean
 */
function is_animated_gif($filename) {
    if(!($fh = @fopen($filename, 'rb')))
        return false;
    $count = 0;
    //an animated gif contains multiple "frames", with each frame having a
    //header made up of:
    // * a static 4-byte sequence (\x00\x21\xF9\x04)
    // * 4 variable bytes
    // * a static 2-byte sequence (\x00\x2C)

    // We read through the file til we reach the end of the file, or we've found
    // at least 2 frame headers
    while(!feof($fh) && $count < 2) {
        $chunk = fread($fh, 1024 * 100); //read 100kb at a time
        $count += preg_match_all('#\x00\x21\xF9\x04.{4}\x00[\x2C\x21]#s', $chunk, $matches);
    }

    fclose($fh);
    return $count > 1;
} 
//_________________________________________________________________________//
/**
 * Create an image thumbnail. If dont_resize_if_smaller flag is specified and the source image is smaller than the specified
 * thumbnail width and height then it will just be copied. Quality is a number between 1 - 100
 * @param string $src
 * @param string $dest
 * @param int $thumb_width
 * @param int $thumb_height
 * @param bool $dont_resize_if_smaller
 * @param int $quality
 * @return boolean
 */
function create_image_thumbnail($src, $dest, $thumb_width = 120, $thumb_height = 120, $dont_resize_if_smaller = false, $quality = 100) {

        $temporarily_raised_memory_limit = false;
        $original_memory_limit = ini_get('memory_limit');
        $real_memory_limit = __convert_unit_prefix_to_bytes($original_memory_limit);
        
        $image_info = getimagesize($src);
        $img_width = $image_info[0];
        $img_height = $image_info[1];
        $img_mimetype = $image_info['mime'];
        $required_memory = round( ($img_height * $img_width * 3 * 2));
        
        if ($required_memory > $real_memory_limit)
        {
            
            $temporarily_raised_memory_limit = true;
            $raised_result = @ini_set('memory_limit', $required_memory);
            if (!$raised_result)
                return false;
        }
        // check if it's animated gif and then return
        // hard code limit 3Mb for animated GIF
        if (is_animated_gif($src) && filesize($src) < 3000000) 
        {
            require_once 'gifanim.inc.php';
            $anim_gif_resizer = new \SCHLIX\AnimatedGIFResizer($src, true);
            $result = $anim_gif_resizer->resize($dest, $thumb_width, $thumb_height, true, false);
            if ($temporarily_raised_memory_limit)
                @ini_set('memory_limit', $original_memory_limit);
            return $result;
            
        }
        switch ($img_mimetype) {
            case 'image/jpeg':$base_img = @ImageCreateFromJPEG($src);
                break;
            case 'image/gif':$base_img = @ImageCreateFromGIF($src);
                break;
            case 'image/png':$base_img = @ImageCreateFromPNG($src);
                break;
            default:$base_img = null;
                break;
        }
        
        if ($dont_resize_if_smaller && ($thumb_width >= $img_width) && ($thumb_height >= $img_height))
        {
            return copy($src, $dest);
        }

        $ext = pathinfo($src, PATHINFO_EXTENSION);
        $new_ext = pathinfo($dest, PATHINFO_EXTENSION);

        if ($base_img == null)
        {
            if ($temporarily_raised_memory_limit)
                @ini_set('memory_limit', $original_memory_limit);
            return false; 
        }
        
        // Work out which way it needs to be resized
        $img_width_per = $thumb_width / $img_width;
        $img_height_per = $thumb_height / $img_height;

        if ($img_width_per <= $img_height_per || $img_width > $thumb_width) {
            $thumb_width = $thumb_width;
            $thumb_height = intval($img_height * $img_width_per);
        } else {
            $thumb_width = intval($img_width * $img_height_per);
            $thumb_height = $thumb_height;
        }
        if ($img_width <= $thumb_width && $img_height <= $thumb_height) {

            $thumb_height = $img_height;
            $thumb_width = $img_width;
        }
        if ($thumb_width == 0)
            $thumb_width++;
        if ($thumb_height == 0)
            $thumb_height++;
        $thumb_img = ImageCreateTrueColor($thumb_width, $thumb_height);
        if ($image_info[2] == IMAGETYPE_GIF || $image_info[2] == IMAGETYPE_PNG) {
            imageAlphaBlending($thumb_img, false);
            imageSaveAlpha($thumb_img, true);
            $transparent = imagecolorallocatealpha($thumb_img, 255, 255, 255, 127);
            @imageFilledRectangle($thumb_img, 0, 0, $thumb_width, $thumb_height, $transparent);
        }

        ImageCopyResampled($thumb_img, $base_img, 0, 0, 0, 0, $thumb_width, $thumb_height, $img_width, $img_height);

        
        switch (strtolower($new_ext))
        {
            case 'png':
                imageAlphaBlending($thumb_img, false);
                imageSaveAlpha($thumb_img, true);
                $result = ImagePNG($thumb_img, $dest, 8);
                break;
                
            case 'jpeg':
            case 'jpg':$result = ImageJPEG($thumb_img, $dest, $quality);break;
            case 'gif':$result = ImageGIF($thumb_img, $dest);break;
        }
        // Clean up our images
        ImageDestroy($base_img);
        ImageDestroy($thumb_img);
        if ($temporarily_raised_memory_limit)
            @ini_set('memory_limit', $original_memory_limit);
        
        return $result;
}


/**
 * Create .htaccess and web.config to restrict folder viewing
 * @param string $folder
 * @return boolean
 */
function create_htaccess_restrict_view($folder) {
    $htaccess = "AuthType Basic\n" .
        "AuthName \"Restricted Area\"\n" .
        "require valid-user\n";


    $webconfig = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" .
        "<configuration>\n" .
        "<system.web>\n" .
        "<authorization>\n" .
        "<deny users =\"*\" />\n" .
        "</authorization>\n" .
        "</system.web>\n" .
        "</configuration>\n";

    if (is_dir($folder)) {
        $filename1 = remove_multiple_slashes($folder . '/.htaccess');
        $filename2 = remove_multiple_slashes($folder . '/web.config');
        return (file_put_contents($filename1, $htaccess) && file_put_contents($filename2, $htaccess));
    } else
        return FALSE;
}

function create_directory_if_not_exists($main_folder, $log_if_error = false)
{
    $result = true;
    if (!is_dir($main_folder)) {
        if (!@mkdir($main_folder, 0755, true)) {
            $result = false;
            if ($log_if_error)
            {
                global $SystemLog;
                if ($SystemLog) // in case it has not been instantiated
                    $SystemLog->error("Unable to create folder {$main_folder}");
            }
        }
    }
    return $result;
}

/**
 * Create sub-site folder skeleton. Prefix is usually SCHLIX_SITE_HTTPBASE
 * @param string $prefix
 * @param string $sitename
 * @return boolean
 */
function create_site_skeleton($prefix, $sitename) {

    $subfolders = array
      ('/apps',
      '/blocks',
      '/macros',
      '/themes',
      '/wysiwygeditors',
      '/wysiwygeditors/tinymce4',
      '/wysiwygeditors/tinymce4/plugins',
      '/wysiwygeditors/ckeditor4',
      '/wysiwygeditors/ckeditor4/plugins',
      '/data', // for private stuff
      '/data/user',
      '/data/user/avatars',
      '/data/user/avatars/original',
      '/data/user/avatars/large',
      '/data/user/avatars/medium',
      '/data/user/avatars/small',
      '/data/user/private',
      '/data/user/public',
      '/data/private',
      '/data/private/mail',
      '/data/private/logs',
      '/data/private/downloads',
      '/data/private/attachments',
      '/data/private/tmp',
      '/data/public',
      '/cache',
      '/cache/generic',
      '/cache/page',
      '/cache/sql',
      '/cache/thumbnails',
      '/media',
      '/media/articles',      
      '/media/images',
      '/media/images/clippings',      
      '/media/downloads',
      '/media/pdf',
      '/media/docs',
      '/media/imagegallery',
      '/media/audios',
      '/media/videos',
      '/media/slideshow');


    if (empty($sitename))
        return FALSE;
    
    $main_folder = $prefix . '/web/' . $sitename;
    $result = create_directory_if_not_exists($main_folder);
    if ($result)
    {
        foreach ($subfolders as $subfolder) {
            $target = $main_folder . $subfolder;
            $result = create_directory_if_not_exists($target);
        }
    }
    create_htaccess_restrict_view($main_folder . '/cache/sql');
    create_htaccess_restrict_view($main_folder . '/cache/page');
    create_htaccess_restrict_view($main_folder . '/cache/generic');
    create_htaccess_restrict_view($main_folder . '/data/private');
    return $result;
}
//_________________________________________________________________________//
/**
 * Returns a UNIX-style file path (with forward slash /) and remove
 * multiple slashes.
 * @param string $path
 * @return string
 */
function normalize_file_path($path) {
    return remove_multiple_slashes(str_replace('\\', '/', $path));
}

//_________________________________________________________________________//
/**
 * This function is an alias of remove_prefix_from_string. This function is used to determine
 * the base path of a URL. e.g. $filename = /var/www/html, $prefix = /var
 * then it would return /www/html
 * @param string $filename
 * @param string $prefix
 * @return string
 */
function remove_prefix_from_file_name($filename, $prefix) {
    return remove_prefix_from_string($filename, $prefix);
}
/**
 * Remove leading prefix from a string. 
 * @param string $str
 * @param string $prefix
 * @return string
 */

function remove_prefix_from_string($str, $prefix) {
    $s = strpos($str, $prefix);
    if (($s == 0) && ($s !== false)) {
        $prefix_length = strlen($prefix);
        return substr($str, $prefix_length, strlen($str) - $prefix_length);
    }
}

/**
 * This is a function designed to replace the base path. Not intended to be 
 * called during page exection. However $SystemDB must be initialized.
 * Please call setupSchlixRuntime() before executing this function
 * @internal
 * @ignore
 * @global \SCHLIX\cmsDatabase $SystemDB
 * @param string $str_old_httpbase
 * @param string $str_new_httpbase
 */
function __force_change_site_httpbase($str_old_httpbase, $str_new_httpbase)
{    
    global $SystemDB;

    $str_old_httpbase = remove_multiple_slashes(trim($str_old_httpbase));
    $str_new_httpbase = remove_multiple_slashes(trim($str_new_httpbase));
    if ($str_old_httpbase == '/' || $str_old_httpbase == '')
    {
        // because the one replaced is "/" only, then add trailing slash
        // for the replacement
        $str_old_httpbase = '/';
        $str_new_httpbase = remove_multiple_slashes($str_new_httpbase.'/');
    } else
    {
        if ($str_new_httpbase != '' && $str_new_httpbase != '/')
        {
            // make sure there's no trailing slash for non "/"
            $str_new_httpbase = rtrim($str_new_httpbase,'/');
        }
    }
    $arr_search = [
        'href="'.$str_old_httpbase => 'href="'.$str_new_httpbase,
        'src="'.$str_old_httpbase => 'src="'.$str_new_httpbase,
    ];
    
    $result = $SystemDB->getQueryResultArray("SHOW TABLES");
    $all_tables = array_map('array_values', $result);
    foreach ($all_tables as $tbl) {
        $ticked_table_name = "`{$tbl[0]}`";
        $sql = "SHOW COLUMNS FROM {$ticked_table_name}";
        $result = $SystemDB->getQueryResultArray($sql);
        $__schlix_table_columns[$tbl[0]] = array_column($result, 'Field');

        if (in_array('summary', $__schlix_table_columns[$tbl[0]])) {
            foreach ($arr_search as $str_search => $str_replace)
                    $SystemDB->searchAndReplace($tbl[0], 'summary', $str_search, $str_replace);
        }
        if (in_array('description', $__schlix_table_columns[$tbl[0]])) {
            foreach ($arr_search as $str_search => $str_replace)
                $SystemDB->searchAndReplace($tbl[0], 'description', $str_search, $str_replac2);
        }
    }
    // now change the menu
    $menu = new \App\Menus();
    $menu->refreshAllMenuLinks(true); // force refresh

}
//_________________________________________________________________________//
/**
 * Recursive delete a directory
 * @internal
 * @ignore
 * @param string $dir
 * @return bool
 */
function __del_tree($dir) {
    if ($dir == '/')
        return false;
    if (!file_exists($dir))
        return true;
    if (!is_dir($dir))
        return @unlink($dir);

    $iterator = new RecursiveDirectoryIterator($dir, FilesystemIterator::SKIP_DOTS);
    foreach (new RecursiveIteratorIterator($iterator, RecursiveIteratorIterator::CHILD_FIRST) as $file) {
        if ($file->isDir())
            @rmdir($file->getPathname());
        else
            @unlink($file->getPathname());
    }
    return @rmdir($dir);
}

//_________________________________________________________________________//
/**
 * Create an image with watermark (usually PNG watermark)
 * @param string $fileSrc
 * @param string $thumbDest
 * @param string $watermarkSrc
 * @param int $thumb_width
 * @param int $thumb_height
 * @return boolean
 */
function create_image_thumbnail_with_watermark($fileSrc, $thumbDest, $watermarkSrc, $thumb_width = 120, $thumb_height = 120) {
    $ext = strtolower(substr($fileSrc, strrpos($fileSrc, ".")));
    if ($ext == ".png") {
        $base_img = ImageCreateFromPNG($fileSrc);
    } else if (($ext == ".jpeg") || ($ext == ".jpg")) {
        $base_img = ImageCreateFromJPEG($fileSrc);
    } else if (($ext == ".gif")) {
        $base_img = ImageCreateFromGIF($fileSrc);
    }

    // If the image is broken, skip it?
    if (!$base_img)
        return false;

    $watermark = ImageCreateFromPNG($watermarkSrc);
    $watermark_width = imagesx($watermark);
    $watermark_height = imagesy($watermark);
    $image = imagecreatetruecolor($watermark_width, $watermark_height);

    // Get image sizes from the image object we just created
    $img_width = imagesx($base_img);
    $img_height = imagesy($base_img);


    // Work out which way it needs to be resized
    $img_width_per = $thumb_width / $img_width;
    $img_height_per = $thumb_height / $img_height;

    if ($img_width_per <= $img_height_per || $img_width > $thumb_width) {
        $thumb_width = $thumb_width;
        $thumb_height = intval($img_height * $img_width_per);
    } else {
        $thumb_width = intval($img_width * $img_height_per);
        $thumb_height = $thumb_height;
    }
    if ($img_width <= $thumb_width && $img_height <= $thumb_height) {

        $thumb_height = $img_height;
        $thumb_width = $img_width;
    }
    $thumb_img = ImageCreateTrueColor($thumb_width, $thumb_height);
    if ($ext == ".png") {
        $white = imagecolorallocate($thumb_img, 255, 255, 255);
        imagefill($thumb_img, 0, 0, $white);
    }

    ImageCopyResampled($thumb_img, $base_img, 0, 0, 0, 0, $thumb_width, $thumb_height, $img_width, $img_height);

    $dest_x = ($thumb_width - $watermark_width) / 2;
    $dest_y = ($thumb_height - $watermark_height) / 2;
    imagecopy($thumb_img, $watermark, $dest_x, $dest_y, 0, 0, $watermark_width, $watermark_height);

    if ($ext == ".png") {
        //	$background = imagecolorallocate($base_img, 122, 122, 122);
        //	imagecolortransparent($thumb_img, $background);
        //imagefilledrectangle ($thumb_img, 0, 0, $thumb_width, $thumb_height, $whitecolor);
        ImagePNG($thumb_img, $thumbDest);
    } else if (($ext == ".jpeg") || ($ext == ".jpg")) {
        ImageJPEG($thumb_img, $thumbDest);
    }

    // Clean up our images
    ImageDestroy($base_img);
    ImageDestroy($thumb_img);
}

/**
 * Converts unit in Giga, mega, or kilo to bytes e.g. 1M, 2G, 30K etc
 * @internal
 * @param string $val
 * @return int
 */
function __convert_unit_prefix_to_bytes($val) {
    $val = trim($val);
    $last = strtoupper($val[strlen($val) - 1]);
    switch ($last) {
        case 'G':$val *= 1024;
        case 'M':$val *= 1024;
        case 'K':$val *= 1024;
    }
    return $val;
}

/**
 * Returns max upload size in bytes
 * @return int
 */
function get_real_max_upload_size() {
    

    $max_upload = __convert_unit_prefix_to_bytes(ini_get('upload_max_filesize'));
    $max_post = __convert_unit_prefix_to_bytes(ini_get('post_max_size'));
    $memory_limit = __convert_unit_prefix_to_bytes(ini_get('memory_limit'));
    return min($max_upload, $max_post, $memory_limit);
}

/**
 * Get the class and function name that called the current method
 * @return string
 */
function get_calling_class_function() {

    //get the trace
    $trace = debug_backtrace();

    // Get the class that is asking for who awoke it
    $class = $trace[1]['class'];

    // +1 to i cos we have to account for calling this function
    for ( $i=1; $i<___c( $trace ); $i++ ) {
        if ( isset( $trace[$i] ) ) // is it set?
             if ( $class != $trace[$i]['class'] ) // is it a different class\
             {
                 return array('class'=>$trace[$i]['class'],'function' => $trace[$i]['function']);
             }
    }
}

/**
 * Returns $_GET variable as a boolean
 * @param string $varname
 * @return bool
 */
function fget_bool($varname) {
    return \SCHLIX\cmsHttpInputFilter::bool($_GET, $varname);
}

/**
 * Returns $_POST variable as a boolean
 * @param string $varname
 * @return bool
 */
function fpost_bool($varname) {
    return \SCHLIX\cmsHttpInputFilter::bool($_POST, $varname);
}

/**
 * Returns $_GET variable as  alphanumeric
 * @param string $varname
 * @param int $limit
 * @return string|int
 */
function fget_alphanumeric($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::alphanumeric($_GET, $varname, $limit);
}

/**
 * Returns $_POST variable as  alphanumeric
 * @param string $varname
 * @return int
 */
function fpost_alphanumeric($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::alphanumeric($_POST, $varname, $limit);
}


/**
 * Returns $_GET variable as an integer
 * @param string $varname
 * @return int
 */
function fget_int($varname) {
    return \SCHLIX\cmsHttpInputFilter::int($_GET, $varname);
}

/**
 * Returns $_GET variable as an unsigned integer
 * @param string $varname
 * @return int
 */
function fget_uint($varname) {
    return \SCHLIX\cmsHttpInputFilter::int($_GET, $varname);
}

/**
 * Returns $_POST variable as an integer
 * @param string $varname
 * @return int
 */
function fpost_int($varname) {
    return \SCHLIX\cmsHttpInputFilter::int($_POST, $varname);
}


/**
 * Returns $_POST variable as an unsigned integer
 * @param string $varname
 * @return int
 */
function fpost_uint($varname) {
    return \SCHLIX\cmsHttpInputFilter::uint($_POST, $varname);
}

/**
 * Returns $_GET variable as a float number
 * @param string $varname
 * @return int
 */
function fget_float($varname) {
    return \SCHLIX\cmsHttpInputFilter::float($_GET, $varname);
}

/**
 * Returns $_POST variable as a float number
 * @param string $varname
 * @return int
 */
function fpost_float($varname) {
    return \SCHLIX\cmsHttpInputFilter::float($_POST, $varname);
}
/**
 * Returns $_GET variable as adouble
 * @param string $varname
 * @return int
 */
function fget_double($varname) {
    return \SCHLIX\cmsHttpInputFilter::double($_GET, $varname);
}

/**
 * Returns $_POST variable as double
 * @param string $varname
 * @return int
 */
function fpost_double($varname) {
    return \SCHLIX\cmsHttpInputFilter::double($_POST, $varname);
}

/**
 * Returns the value of $_GET[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fget_string($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string($_GET, $varname, true, $limit);
}

/**
 * Returns the value of $_POST[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fpost_string($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string($_POST, $varname, true, $limit);
}

/**
 * Returns the value of $_GET[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fget_string_notags($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string_notags($_GET, $varname, $limit);
}

/**
 * Returns the value of $_POST[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fpost_string_notags($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string_notags($_POST, $varname, $limit);
}


/**
 * Returns the value of $_GET[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fget_string_noquotes_notags($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string_noquotes_notags($_GET, $varname, $limit);
}

/**
 * Returns the value of $_POST[$varname], limited to $limit
 * @param string $varname
 * @param int $limit
 * @return string
 */
function fpost_string_noquotes_notags($varname, $limit = 0) {
    return \SCHLIX\cmsHttpInputFilter::string_noquotes_notags($_POST, $varname, $limit);
}

/**
 * Returns a sanitized filename from $_GET[$varname]
 * @param string $varname
 * @return string
 */
function fget_filename_only($varname) {
    return \SCHLIX\cmsHttpInputFilter::filename_only($_GET, $varname);
}

/**
 * Returns a sanitized filename from $_POST[$varname]
 * @param string $varname
 * @return string
 */
function fpost_filename_only($varname) {
    return \SCHLIX\cmsHttpInputFilter::filename_only($_POST, $varname);
}

/**
 * Returns a sanitized full path from $_GET[$varname]
 * @param string $varname
 * @return string
 */
function fget_filename_fullpath($varname) {
    return SCHLIX\cmsHttpInputFilter::filename_fullpath($_GET, $varname);
}

/**
 * Returns a sanitized full path from $_POST[$varname]
 * @param string $varname
 * @return string
 */
function fpost_filename_fullpath($varname) {
    return SCHLIX\cmsHttpInputFilter::filename_fullpath($_POST, $varname);
}

/**
 * Returns array from $_POST[$varname]
 * @param string $varname
 * @return array
 */
function fpost_array_any($varname)
{
    return SCHLIX\cmsHttpInputFilter::array_any($_POST, $varname);
}

/**
 * Returns array of int from $_POST[$varname]
 * @param string $varname
 * @return array
 */
function fpost_array_int($varname)
{
        return SCHLIX\cmsHttpInputFilter::array_int($_POST, $varname);
}
